@@ -1,6 +1,9 @@
+import secrets
 from django.db import models
 from pretix.base.models import OrderPosition
 
+def generate_totp_secret():
+    return secrets.token_bytes(20)
 
 class AppleWalletPass(models.Model):
     order_position = models.OneToOneField(OrderPosition, on_delete=models.CASCADE, related_name="apple_wallet_pass", db_index=True)
@@ -27,4 +30,22 @@ class ApplePassRegistration(models.Model):
     device = models.ForeignKey(AppleDevice, on_delete=models.CASCADE, related_name="registrations", db_index=True)
 
     class Meta:
-        unique_together = (("order_position", "device"),)
\ No newline at end of file
+        unique_together = (("order_position", "device"),)
+
+class GoogleEventLog(models.Model):
+    class_id = models.CharField(max_length=255)
+    object_id = models.CharField(max_length=255)
+    nonce = models.CharField(max_length=255)
+    event_type = models.CharField(max_length=64, choices=(
+        ("save", "Add"),
+        ("del", "Delete"),
+    ))
+    timestamp = models.DateTimeField(auto_now_add=True)
+
+    class Meta:
+        unique_together = (("class_id", "object_id", "nonce"),)
+        ordering = ("-timestamp",)
+
+class OrderPositionTotp(models.Model):
+    order_position = models.OneToOneField(OrderPosition, on_delete=models.CASCADE, related_name="totp", db_index=True)
+    totp_key = models.BinaryField(default=generate_totp_secret)
\ No newline at end of file
