@@ -3,12 +3,11 @@ import json
 import typing
 import collections
 import urllib.parse
-import urllib3
-import idna
 import pytz
 from django import forms
 from django.conf import settings
 from django.contrib.staticfiles import finders
+from django.core.files import File
 from django.core.files.storage import default_storage
 from django.core.validators import RegexValidator
 from django.utils import timezone
@@ -17,20 +16,13 @@ from django.utils.translation import gettext_lazy as _
 from pretix.base.models import Order, OrderPosition
 from pretix.base.ticketoutput import BaseTicketOutput
 from pretix.multidomain.urlreverse import build_absolute_uri
-from . import pkpass, barcode, models
+from . import pkpass, barcode, models, vas, utils
 from .forms import PNGImageField
 
 
-def idna_encode_url(url: str):
-    parts = urllib3.util.parse_url(url)
-    host = idna.encode(parts.host).decode()
-    new_url = urllib3.util.Url(parts.scheme, parts.auth, host, parts.port, parts.path, parts.query, parts.fragment)
-    return new_url.url
-
-
 class AppleWalletOutput(BaseTicketOutput):
     identifier = "apple-wallet-uic"
-    verbose_name = "Apple Wallet - UIC Barcode"
+    verbose_name = _("Apple Wallet")
     download_button_icon = "fa-mobile"
     download_button_text = _("Apple Wallet")
     multi_download_enabled = True
@@ -39,6 +31,7 @@ class AppleWalletOutput(BaseTicketOutput):
         super().__init__(*args, **kwargs)
         self.signer = pkpass.get_signer()
         self.barcode_generator = barcode.UICBarcodeGenerator(self.event)
+        self.vas_generator = vas.VASDataGenerator(self.event)
 
     @cached_property
     def module_generators(self) -> list:
@@ -165,7 +158,7 @@ class AppleWalletOutput(BaseTicketOutput):
         pass_serial = f"{order.event.organizer.slug}-{position.code}"
         pass_json = {
             "formatVersion": 1,
-            "organizationName": self.signer.pass_signer_name,
+            "organizationName": str(event.organizer.name),
             "passTypeIdentifier": self.signer.pass_type_id,
             "teamIdentifier": self.signer.team_id,
             "serialNumber": pass_serial,
@@ -174,7 +167,7 @@ class AppleWalletOutput(BaseTicketOutput):
             "suppressStripShine": True,
             "suppressHeaderDarkening": True,
             "locations": [],
-            "webServiceURL": idna_encode_url(urllib.parse.urljoin(settings.SITE_URL, "/api/apple_wallet")),
+            "webServiceURL": utils.idna_encode_url(urllib.parse.urljoin(settings.SITE_URL, "/api/apple_wallet")),
             "authenticationToken": position.web_secret,
             "eventTicket": {
                 "headerFields": [],
@@ -188,19 +181,27 @@ class AppleWalletOutput(BaseTicketOutput):
             "voided": order.status != Order.STATUS_PAID,
         }
 
-        op_secret = self.barcode_generator.generate_barcode(position)
-        if self.event.settings.uic_barcode_encoding == "b45":
-            pass_json["barcodes"].append({
-                "format": "PKBarcodeFormatQR",
-                "message": op_secret.decode("utf-8"),
-                "messageEncoding": "utf-8",
-                "altText": position.secret,
-            })
+        if self.event.settings.ticket_secret_generator == "uic-barcodes":
+            op_secret = self.barcode_generator.generate_barcode(position)
+            if self.event.settings.uic_barcode_encoding == "b45":
+                pass_json["barcodes"].append({
+                    "format": "PKBarcodeFormatQR",
+                    "message": op_secret.decode("utf-8"),
+                    "messageEncoding": "utf-8",
+                    "altText": position.secret,
+                })
+            else:
+                pass_json["barcodes"].append({
+                    "format": "PKBarcodeFormatAztec",
+                    "message": op_secret.decode("iso-8859-1"),
+                    "messageEncoding": "iso-8859-1",
+                    "altText": position.secret,
+                })
         else:
             pass_json["barcodes"].append({
-                "format": "PKBarcodeFormatAztec",
-                "message": op_secret.decode("iso-8859-1"),
-                "messageEncoding": "iso-8859-1",
+                "format": "PKBarcodeFormatQR",
+                "message": position.secret,
+                "messageEncoding": "utf-8",
                 "altText": position.secret,
             })
 
@@ -214,6 +215,7 @@ class AppleWalletOutput(BaseTicketOutput):
             date_to = event.date_to.astimezone(tz)
         else:
             date_to = None
+        date_admission = event.date_admission.astimezone(tz) if event.date_admission else None
 
         if position.item.admission:
             pass_json["eventTicket"]["auxiliaryFields"].append({
@@ -228,7 +230,7 @@ class AppleWalletOutput(BaseTicketOutput):
         if date_to:
             pass_json["expirationDate"] = date_to.isoformat()
             pass_json["relevantDates"].append({
-                "startDate": date_from.isoformat(),
+                "startDate": date_admission.isoformat() if date_admission else date_from.isoformat(),
                 "endDate": date_to.isoformat(),
             })
             if position.item.admission:
@@ -242,7 +244,17 @@ class AppleWalletOutput(BaseTicketOutput):
                 })
         else:
             pass_json["relevantDates"].append({
-                "date": date_from.isoformat(),
+                "date": date_admission.isoformat() if date_admission else date_from.isoformat(),
+            })
+
+        if position.item.admission and event.date_admission:
+            pass_json["eventTicket"]["headerFields"].append({
+                "key": "ev-admission",
+                "label": "Admission",
+                "dateStyle": "PKDateStyleMedium",
+                "timeStyle": "PKDateStyleShort",
+                "value": event.date_admission.astimezone(tz).isoformat(),
+                "ignoresTimeZone": True,
             })
 
         pass_json["eventTicket"]["primaryFields"].append({
@@ -260,14 +272,6 @@ class AppleWalletOutput(BaseTicketOutput):
             "value": product_name,
         })
 
-        pass_json["eventTicket"]["headerFields"].append({
-            "key": "ev-date",
-            "value": event.date_from.astimezone(tz).isoformat(),
-            "dateStyle": "PKDateStyleShort",
-            "timeStyle": "PKDateStyleShort",
-            "ignoresTimeZone": True
-        })
-
         if event.geo_lat and event.geo_lon:
             pass_json["locations"].append({
                 "latitude": float(event.geo_lat),
@@ -310,41 +314,41 @@ class AppleWalletOutput(BaseTicketOutput):
         if label_color := self.settings.get("label_color", None):
             pass_json["labelColor"] = label_color
 
-        if icon_file := self.settings.get("icon", None):
+        if icon_file := self.settings.get("icon", None, as_type=File):
             pk_pass.add_file("icon.png", default_storage.open(icon_file.name, "rb").read())
-            if icon_2x_file := self.settings.get("icon_2x", None):
+            if icon_2x_file := self.settings.get("icon2x", None, as_type=File):
                 pk_pass.add_file("icon@2x.png", default_storage.open(icon_2x_file.name, "rb").read())
-            if icon_3x_file := self.settings.get("icon_3x", None):
+            if icon_3x_file := self.settings.get("icon3x", None, as_type=File):
                 pk_pass.add_file("icon@3x.png", default_storage.open(icon_3x_file.name, "rb").read())
         else:
             pk_pass.add_file("icon.png", open(finders.find("pretix_uic_barcode/icon.png"), "rb").read())
 
-        if logo_file := self.settings.get("logo", None):
+        if logo_file := self.settings.get("logo", None, as_type=File):
             pk_pass.add_file("logo.png", default_storage.open(logo_file.name, "rb").read())
-        if logo_2x_file := self.settings.get("logo_2x", None):
+        if logo_2x_file := self.settings.get("logo2x", None, as_type=File):
             pk_pass.add_file("logo@2x.png", default_storage.open(logo_2x_file.name, "rb").read())
-        if logo_3x_file := self.settings.get("logo_3x", None):
+        if logo_3x_file := self.settings.get("logo3x", None, as_type=File):
             pk_pass.add_file("logo@3x.png", default_storage.open(logo_3x_file.name, "rb").read())
 
-        if strip_file := self.settings.get("strip", None):
+        if strip_file := self.settings.get("strip", None, as_type=File):
             pk_pass.add_file("strip.png", default_storage.open(strip_file.name, "rb").read())
-        if strip_2x_file := self.settings.get("strip_2x", None):
+        if strip_2x_file := self.settings.get("strip2x", None, as_type=File):
             pk_pass.add_file("strip@2x.png", default_storage.open(strip_2x_file.name, "rb").read())
-        if strip_3x_file := self.settings.get("strip_3x", None):
+        if strip_3x_file := self.settings.get("strip3x", None, as_type=File):
             pk_pass.add_file("strip@3x.png", default_storage.open(strip_3x_file.name, "rb").read())
 
-        if thumbnail_file := self.settings.get("thumbnail", None):
+        if thumbnail_file := self.settings.get("thumbnail", None, as_type=File):
             pk_pass.add_file("thumbnail.png", default_storage.open(thumbnail_file.name, "rb").read())
-        if thumbnail_2x_file := self.settings.get("thumbnail_2x", None):
+        if thumbnail_2x_file := self.settings.get("thumbnail2x", None, as_type=File):
             pk_pass.add_file("thumbnail@2x.png", default_storage.open(thumbnail_2x_file.name, "rb").read())
-        if thumbnail_3x_file := self.settings.get("thumbnail_3x", None):
+        if thumbnail_3x_file := self.settings.get("thumbnail3x", None, as_type=File):
             pk_pass.add_file("thumbnail@3x.png", default_storage.open(thumbnail_3x_file.name, "rb").read())
 
-        if background_file := self.settings.get("background", None):
+        if background_file := self.settings.get("background", None, as_type=File):
             pk_pass.add_file("background.png", default_storage.open(background_file.name, "rb").read())
-        if background_2x_file := self.settings.get("background_2x", None):
+        if background_2x_file := self.settings.get("background2x", None, as_type=File):
             pk_pass.add_file("background@2x.png", default_storage.open(background_2x_file.name, "rb").read())
-        if background_3x_file := self.settings.get("background_3x", None):
+        if background_3x_file := self.settings.get("background3x", None, as_type=File):
             pk_pass.add_file("background@3x.png", default_storage.open(background_3x_file.name, "rb").read())
 
         for g in self.module_generators:
